import numpy as np
from scipy.interpolate import interp1d

def fluxconserve(wavelength, flux_density, new_wavelength, kind='linear', fill_val=0.0):
    '''Created on Last version on Wed Sep 23 14:33:51 2020

@author: Jean Gomes Copyright (c)

Python function called fluxconserve that interpolates a given flux density
array onto a new wavelength array and adjusts the flux density values to
conserve the total flux density point-by-point.

The fluxconserve function takes four input arguments:

wavelength: A 1D numpy array containing the wavelengths of the original flux density values.

flux_density: A 1D numpy array containing the flux density values corresponding to the wavelength array.

new_wavelength: A 1D numpy array containing the new wavelengths to interpolate the flux_density array onto.

kind: A string or integer specifying the kind of interpolation to use. The default value is 'linear'.

The function first uses the interp1d function from the scipy.interpolate module to interpolate the flux_density array onto the new new_wavelength array using the specified interpolation kind.

Next, the function calculates a scale factor for each point in the
new_wavelength array to adjust the interpolated flux_density values to
conserve the point-by-point flux density. This is done by dividing the
interpolated flux_density by the original flux_density values at the
corresponding wavelength points. Points where either the interpolated or
original flux_density is not finite are excluded from the scale factor
calculation.

Finally, the function returns the adjusted flux density values using the
scale factor, or a fill_val value for any NaN values.

Overall, this function is useful for conserving the total flux density
when interpolating flux density values onto a new wavelength grid, which
is a common task in many areas of astronomy.'''

    # interpolate the flux density to the new wavelength array
    interp_func = interp1d(wavelength, flux_density, kind=kind, bounds_error=False)
    new_flux_density = interp_func(new_wavelength)

    # interpolate the original flux density to the new wavelength array
    orig_flux_density = interp_func(new_wavelength)

    # adjust the interpolated flux density to conserve point-by-point flux density
    scale = np.ones_like(new_flux_density)
    idx = (np.isfinite(new_flux_density) & np.isfinite(orig_flux_density))
    scale[idx] = new_flux_density[idx] / orig_flux_density[idx]
    return np.where( np.isnan(orig_flux_density * scale), fill_val, orig_flux_density * scale )

# kind : str or int, optional Specifies the kind of interpolation as a string
# or as an integer specifying the order of the spline interpolator to use. The
# string has to be one of ‘linear’, ‘nearest’, ‘nearest-up’, ‘zero’,
# ‘slinear’, ‘quadratic’, ‘cubic’, ‘previous’, or ‘next’. ‘zero’, ‘slinear’,
# ‘quadratic’ and ‘cubic’ refer to a spline interpolation of zeroth, first,
# second or third order; ‘previous’ and ‘next’ simply return the previous or
# next value of the point; ‘nearest-up’ and ‘nearest’ differ when
# interpolating half-integers (e.g. 0.5, 1.5) in that ‘nearest-up’ rounds up
# and ‘nearest’ rounds down. Default is ‘linear’.

# generate some test data
#wavelength = np.linspace(4000, 8000, 1000)
#flux_density = np.random.normal(size=1000)
#
## define the new wavelength array to interpolate to
#new_wavelength = np.linspace(1000, 12000, 500)
#
## interpolate the flux density to the new wavelength array while conserving point-by-point flux density
#new_flux_density = interpolate_flux(wavelength, flux_density, new_wavelength)
##new_flux_density = np.where( np.isnan(new_flux_density), 0.0, new_flux_density )
#
#
#print(new_wavelength.size,new_wavelength[0],new_wavelength[-1])
#print(new_wavelength.size,new_flux_density[0],new_flux_density[-1])
#
## plot the results
#import matplotlib.pyplot as plt
#plt.plot(wavelength, flux_density, label='Original')
#plt.plot(new_wavelength, new_flux_density, label='Interpolated')
#plt.legend()
#plt.show()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Jan 30 12:07:21 2021

RESUME : Interpolation script calling Akima Spline in Fortran!
    
Version: v01 beta

@author: Jean Gomes Copyright (c)

@email: jean@iastro.pt

Written: Jean Michel Gomes © Copyright
"""

# Import libraries
# Be careful with importation of pylab - It needs to be first otherwise problems with other features
from pylab import *
import matplotlib.pyplot as plt

import numpy as np

from PyFluxconserving import flib as interp
import scipy.interpolate as interp1

import scipy.interpolate as interpolate
from scipy.interpolate import CubicSpline
from scipy.interpolate import BSpline

# Class PyAkimaSpline
class PyAkimaSpline( object ):
    """Created on 
Last version on Wed Sep 23 14:33:51 2020

@author: Jean Gomes Copyright (c)

*** Number : 005
1) ABS__smooth
2) Akima_Setup
3) Akimainterp
4) AkimaSpline
5) author_AkimaSpline

RESUME : Class for evaluating Akima Spline.
                  Akima Spline interpolation. The Akima spline is a C1
                  differentiable function (that is, has a continuous first derivative)
                  but, in general, will have a discontinuous second derivative at the
                  knot points.

           [1]: Akima, H. (1970). A New Method of Interpolation and Smooth Curve
                  Fitting Based on Local Procedures. Journal of the ACM, 17(4),
                  589-602. Association for Computing
                  Machinery. doi:10.1145/321607.321609

######################################################################
SUBROUTINE ABS__smooth( x,y,delta_x )
abs__smooth in python 

RESUME : Absolute value function with small quadratic parabola-like
                  in the valley so that it is C1 continuous. C1 continuous
                 - Continuous first derivative.
                                                                            
Input                    arguments = 2                                         
Output                 arguments = 1                                         
Optional              arguments = 0                                         
Total number of arguments = 3                                         
  
  INPUT  : 01) x                -> Old x vector (abcissas)               
                  02) delta_x     -> Valley window point to be quadratic           
                                                                            
OUTPUT : 01) y               -> Output value                                  
                                                                            
PYTHON : Python compatibility using f2py revised. Better usage        
                  with numpy.                                                  
                                                                            
Written: Jean Michel Gomes © Copyright ®                              
Checked: Tue May  1 16:09:13 WEST 2012                                
                  Fri Dec 28 14:55:10 WET  2012                                
######################################################################

######################################################################
SUBROUTINE Akima_Setup( xold_vec,yold_vec,nold_vec,p0,p1,p2,p3,delta__x )
akima_setup in python
    
RESUME : Compute the Spline Coefficients for the Akima Spline         
                  function.                                                    
                                                                           
Input                    arguments = 4                                         
Output                 arguments = 4                                         
Optional              arguments = 0                                         
Total number of arguments = 8                                         
                                                                          
   INPUT  : 01) xold_vec -> Original Abscissa                            
                   02) yold_vec -> Original Ordinate                            
                   03) nold_vec -> # of Points                                  
                   04) delta__x  -> Valley window point to be quadratic          
                                                                           
OUTPUT : 01) p0            -> Spline Coefficient                           
                  02) p1            -> Spline Coefficient                           
                  03) p2            -> Spline Coefficient                           
                  04) p3            -> Spline Coefficient                           

PYTHON : Python compatibility using f2py revised. Better usage        
                  with numpy.                                                  
                                                                           
Written: Jean Michel Gomes © Copyright ®                              
Checked: Wed May  2 10:00:52 WEST 2012                                
                  Fri Dec 28 13:53:36 WET  2012                                
                 Sun Mar 10 10:05:03 WET  2013                                
######################################################################  

######################################################################  
SUBROUTINE Akimainterp( xx_value,yy_value,nx_value,xold_vec,nold_vec,p0,p1,                            &
                                                 p2,p3     ,dp0dxold,dp1dxold,dp2dxold,dp3dxold,                                  &
                                                dp0dyold,dp1dyold,dp2dyold,dp3dyold,dy____dx,                                  &
                                                dy_dxold,dy_dyold,Is_Deriv,Is_Index )

Akimainterp in python

RESUME : Evaluate the Akima Spline and its derivatives.  The          
                  Akima spline is a C1 differentiable unction (that is,        
                  has a continuous first derivative) but, in general,                        
                  will have a discontinuous second derivative at the knot      
                  points.                                                      
                                                                           
 Input                   arguments = 16
 Output                arguments = 4
 Optional              arguments = 1
 Total number of arguments = 21 

   INPUT  : 01) xx_value  -> New interpolated x array with points         
                  02) nxyvalue  -> # of elements in xx_value and yy_value       
                  03) xold_vec   -> Old x vector (abcissas)                      
                  04) nold_vec  -> # of elements in xold_vec and yold_vec       
                  05) p0             -> Spline Coefficient                           
                  06) p1             -> Spline Coefficient                          
                  07) p2             -> Spline Coefficient                          
                  08) p3             -> Spline Coefficient                           
                  09) dp0dxold -> Derivatives of coefficients                  
                  10) dp1dxold -> Derivatives of coefficients                  
                  11) dp2dxold -> Derivatives of coefficients                  
                  12) dp3dxold -> Derivatives of coefficients                  
                  13) dp0dyold -> Derivatives of coefficients                   
                  14) dp1dyold -> Derivatives of coefficients                  
                  15) dp2dyold -> Derivatives of coefficients                  
                  16) dp3dyold -> Derivatives of coefficients                  
                  17) Is_Deriv   -> Is derivative on:1 or off:0                  
                                                                           
OUTPUT : 01) yy_value -> New interpolated yy_value array              
                  02) dy____dx -> Derivative of yy_value w.r.t. xx_value       
                  03) dy_dxold -> Derivative of yy_value w.r.t. yold_vec       
                  04) dy_dyold -> Derivative of yy_value w.r.t. yold_vec       
                                                                           
PYTHON : Python compatibility using f2py revised. Better usage        
                  with numpy.                                                  
                                                                           
Written: Jean Michel Gomes © Copyright ®                              
Checked: Wed May  2 10:00:52 WEST 2012                                
                 Fri Dec 28 13:53:36 WET  2012                                
                 Sun Mar 10 10:05:03 WET  2013                                !
######################################################################  

######################################################################
SUBROUTINE AkimaSpline( xx_value,yy_value,nxyvalue,xold_vec,yold_vec,                                       &
                                                  nold_vec,delta__x,IsKeepOn,verbosity )
akmaspline in python

RESUME : For a given array of values 'xx_value' for the               
              abscissa, then return the ordinate array values of           
              'yy_value' based on Akima Spline interpolation.               
                                                                          
   INPUT  : 01) xx_value  -> New interpolated x array with points        
                  02) nxyvalue  -> # of elements in xx_value and yy_value      
                  03) xold_vec  -> Old x vector (abcissas)                     
                  04) yold_vec  -> Old y vector (ordenadas)                    
                  05) nold_vec  -> # of elements in xold_vec and yold_vec      
                  06) delta__x  -> Value used in the absolute value function   
                                             with small quadratic parabola-like in the   
                                             valley so that it is C1 continuous. C1      
                                             continuous                                  
                  07) verbosity -> Print & Check screen                        
                                                                           
OUTPUT : 01) yy_value -> New interpolated y array with points         
                  02) IsKeepOn -> Flag, if == 0 then there's a problem         
                                                                           
PYTHON : Python compatibility using f2py revised. Better usage        
              with numpy.                                                  
                                                                           
Written: Jean Michel Gomes © Copyright ®                              
Checked: Wed May  2 10:00:52 WEST 2012                                
                 Fri Dec 28 13:53:36 WET  2012                                
                 Sun Mar 10 10:05:03 WET  2013                                
######################################################################  

######################################################################  
SUBROUTINE author_AkimaSpline( a )
author_akimaspline in python
######################################################################  

######################################################################             
                 HOW TO USE
                 
                 i_object = AkimaSpline
                 i_object.AkimaSpline(x_new,x,y,[delta_x],[verbosity])
                 
                 where
                 
                 x_new is the new array of points (abscissa) to be flux-conserved
                 x,y are the original arrays (abscissa and ordinate)
                 
                 [delta_x] => Value used in the absolute value function with small quadratic 
                                       parabola-like in the valley so that it is C1 continuous. 
                                       C1 continuous (continuous first derivative).
                 [verbosity] => verbosity mode [0: No, 1:Yes] 
                 
                 Test ROUTINE
                 
                 i_object = AkimaSpline
                 i_object.TestRoutine()
                 
                 It will produce a plot with the sine interpolation
                 
                 time
                 
                 i_object = AkimaSpline
                 i_object.time()
                 
                 It will test the time it takes to interpolate as a function of elements in the array
######################################################################                  
"""

    def __init__( self,x_new,x,y,delta_x=0.1,tipo=0,verbosity=0 ):
        self.x               = x
        self.y               = y
        self.x_new      = x_new

        self.delta_x = delta_x
        self.verbosity = verbosity
        
        if tipo == 0:
            self.y_new, self.IsKeepOn = interp.akimaspline( self.x_new, self.x, self.y, delta__x=self.delta_x, verbosity=self.verbosity )
            return
        if tipo == 1: # Akima using the Python3 interface
            self.y_new = akima_interp( self.x_new, self.x, self.y )
            return
        if tipo == 2: # Akima using the Scipy interface
            scipy_akima_object = interp1.Akima1DInterpolator(  self.x, self.y )
            self.y_new = scipy_akima_object( self.x_new )
            return
         
        # In case no tipo is chosen, then use tipo == 0
        self.y_new, self.IsKeepOn = interp.akimaspline( self.x_new, self.x, self.y, delta__x=self.delta_x,verbosity=self.verbosity )
        return
        
    def plot( self ):
        Figure = plt.figure( figsize=(12,10),dpi=120,facecolor='w',edgecolor='w' )
        plt.subplots_adjust(bottom=.02, left=.06, right=.95, top=.98, wspace=0.0, hspace=0.0) 
        
        ax = plt.subplot(111)

        # Main plot ##########################################################
        ax1_top = subplot2grid( (20,20), (0,0), colspan=20, rowspan=20 )                                                 # Sets the position and size of the panel for Plot #01
        ax1_top.axis('on')
        ax1_top.axes.get_xaxis().set_visible(True)
    
        ax1_top.plot( self.x,self.y,linewidth=4,color='darkorange' ,label='Original')
        ax1_top.plot( self.x_new,self.y_new,linewidth=3,color='#1f77b4' ,label='Interpolation')
        
        ax1_top.legend()
        
        ax1_top.set_xlabel('x')
        ax1_top.set_ylabel('y(x)')
        # Main plot ##########################################################
                                  
    def TestRoutine( delta_x=0.1 ):
        print("... Create a sine wave function")
        print("... x and y values for x with equal step size")
        
        x = np.arange(0,2.0*np.pi,1.0)
        y = np.sin(x)

        print("... step x: {0:}".format(x[1]-x[0]))

        # No verbosity 
        verbosity = 0
        
        print("... x and y values for x with unequal step size")
        
        #x_new = [x[0]-0.1,x[0],0.5,1.0,2.3,3.22222,4.66,5.2,6.0,x[-1],x[-1]+0.1]
        x_new = [x[0]-0.2,x[0]-0.1,x[0],0.5,1.0,1.5,2.3,3.22222,4.66,5.2,6.0,x[-1],x[-1]+0.1,x[-1]+0.5]
        x_new = np.array( x_new, dtype=float )
        x_new = np.unique(x_new)
        y_new = np.sin(x_new)
        
        dxnew = x_new[1:] - x_new[:-1]
        
        print("... step x_new: {0:}".format(dxnew))
                
        #x_new = [x[0]-0.5,x[0]-0.1,x[0],0.5,1.0,1.5,2.3,3.22222,4.66,5.2,6.0,x[-1],x[-1]+0.1,x[-1]+0.5]
        
        x_interp          = np.arange(x[0]-0.5,x_new[-1]+0.5,0.01)
        #x_interp = np.array( [-0.7,-0.7,-0.7,-0.7,-0.7099999,0.2,0.3,0.3,0.9,1.2,1.2,1.5,2.3,2.3,3.4,3.5], dtype=float )
        x_new_interp = x_interp
    
        print("... Interpolation of array: {0:}".format([x_interp[0],x_interp[-1]]))

        # Calling directly the f2py subroutine
        y_interp, IsKeepOn = interp.akimaspline( x_interp, x, y, delta__x=delta_x, verbosity=verbosity )
        y_new_interp, IsKeepOn = interp.akimaspline( x_new_interp, x_new, y_new, delta__x=delta_x, verbosity=verbosity )

        #print(y_new_interp)

        # Akima using the Python3 interface
        y_interp_Akima = akima_interp( x_interp, x, y, delta_x=delta_x )
        y_new_interp_Akima = akima_interp( x_new_interp, x_new, y_new, delta_x=delta_x )

        # From scipy Akima interpolator
        scipy_akima_object = interp1.Akima1DInterpolator( x,y )
        y_interp_scipy = scipy_akima_object( x_interp )
        
        # Transform into unique because scipy does not accept repetitive values
        x_new_unique = np.unique(x_new)
        y_new_unique = np.sin(x_new_unique)
        scipy_akima_object = interp1.Akima1DInterpolator( x_new_unique,y_new_unique )
        y_new_interp_scipy = scipy_akima_object( x_new_interp )

        #print(x_new)
        #print(x_new_unique)
        #print(x_new_interp)
        #for i in enumerate(x_new_interp):
         #   print(x_new_interp[i[0]],y_new_interp_Akima[i[0]])
        
        #print(x_new,y_new)
        #print(y_new_scipy)
        #print("... Interpolation of array: {0:}".format(x_new))
        
        Figure = plt.figure( figsize=(12,10),dpi=120,facecolor='w',edgecolor='w' )
        plt.subplots_adjust(bottom=.02, left=.06, right=.95, top=.98, wspace=0.0, hspace=0.0) 
        
        ax = plt.subplot(111)

        # Top plot ###########################################################
        ax1_top = subplot2grid( (20,20), (0,0), colspan=20, rowspan=10 )                                                 # Sets the position and size of the panel for Plot #01
        ax1_top.axis('on')
        ax1_top.axes.get_xaxis().set_visible(False)

        ylimmin = -1.2
        ylimmax = max( y_interp ) * 1.2
        #ax1_top.set_xlim( xmin,xmax )
        ax1_top.set_ylim( ylimmin,ylimmax )

        # Original values
        ax1_top.plot(x,y,color='darkorange',linewidth=10,label='Original')
        
        # Interpolated function Fortran - f2py
        ax1_top.plot(x_interp, y_interp, color='blue',label='Akima Interpolation',linewidth=10,alpha=0.3)

        # Interpolated function Python3 interface
        ax1_top.plot(x_interp, y_interp_Akima, color='magenta',linestyle='--',linewidth=4,label='Akima Python3 Interface')

        # Interpolated function using scipy
        ax1_top.plot(x_interp, y_interp_scipy,color='black',linewidth=1.2,label='Akima Scipy')

        ax1_top.legend()
        ax1_top.set_xlabel("Radians")
        ax1_top.set_ylabel("f(x) = sin(x)")
        # Top plot ###########################################################

        # Bottom plot ########################################################
        ax2_top = subplot2grid( (20,20), (10,0), colspan=20, rowspan=9 )                                                # Sets the position and size of the panel for Plot #02
        ax2_top.axis('on')
        
        # Original Different Point Values
        ax2_top.plot(x_new,y_new,color='darkorange',label='Original Diff Points',linewidth=10)
        
        # Interpolated function Fortran - f2py
        ax2_top.plot(x_new_interp, y_new_interp, color='blue',label='Akima Interpolation',linewidth=10,alpha=0.3)
        
        # Interpolated function Python3 interface
        ax2_top.plot(x_new_interp, y_new_interp_Akima, color='magenta',linestyle='--',linewidth=4,label='Akima Python3 Interface')

        # Interpolated function using scipy
        ax2_top.plot(x_new_interp, y_new_interp_scipy,color='black',linewidth=1.2,label='Akima Scipy')
        
        ax2_top.legend()
        ax2_top.set_xlabel("Radians")
        ax2_top.set_ylabel("f(x) = sin(x)")
        
        plt.show()
        # Bottom plot ########################################################

    def time():
        print("... Create a sine wave function")
        print("... x and y values for x with equal step size")
     
        x = np.arange(0,2.3*np.pi,0.05)
        y = np.sin(x)
     
        print("... step x: {0:}".format(x[1]-x[0]))

        # No verbosity 
        verbosity = 0
     
        delta_x = 0.1

        #i_ = 25

        #sizepoints = np.zeros(i_)
        #timearray  = np.zeros([i_,3])

        np.random.seed(10)
        
        #x_interp_ = np.random.normal(0.0,1.0,2**i_)
        #x_interp_ = np.unique(x_interp_)
        
        val1 = np.arange(1,10,1)
        val2 = np.arange(10,100,2)
        val3 = np.arange(100,1000,10)
        val4 = np.arange(1000,10000,100)
        val5 = np.arange(10000,100000,1000)
        val6 = np.arange(100000,1000000,10000)
        val7 = np.arange(1000000,10000000,1000000)

        val_array = np.concatenate([val1,val2,val3,val4,val5,val6,val7])
        val_array = np.unique(val_array)
        
        #print(val_array)
        #return
    
        x_interp_ = 0. + 2.3*np.pi * (  np.random.normal(0.0,1.0,val_array[-1])  )
        x_interp_ = np.unique(x_interp_)
        
        i_ = val_array.size
        sizepoints = np.zeros(i_)
        
        population = 10
        timearray  = np.zeros([i_,3,population])
        timearray_average  = np.zeros([i_,3])
        standard_deviation = np.zeros([i_,3])
        
        value = 1
        for i in enumerate(val_array):
            
            for pop in range(0,population):
                value = val_array[i[0]]
                x_interp = x_interp_[0:value]
        
                print("... iteration: {}".format(i[0]))
                print("... Interpolation of array: {0:} {1:} with size == {2:}".format(x_interp[0],x_interp[-1],x_interp.size))

                # Calling directly the f2py subroutine
                auxil1time = time.time()
                #y_interp,IsKeepOn = interp.spline3darr(x_interp,x,y,verbosity=verbosity)
                y_interp, IsKeepOn = interp.akimaspline( x_interp, x, y, delta__x=delta_x, verbosity=verbosity )
                
                # Calling Akima Spline interpolation
                auxil2time = time.time()
                #y_interp_Akima,IsKeepOn = interp1.akimaspline( x_interp, x, y, delta__x=delta_x, verbosity=verbosity )
                scipy_akima_object = interpolate.Akima1DInterpolator( x,y )
                y_interp_scipy = scipy_akima_object( x_interp )
                
                # Cubic Spline using scipy
                auxil3time = time.time()
                cs          = CubicSpline( x,y )
                y_interp_scipy = cs(x_interp)
                
                auxil4time = time.time()
                
                sizepoints[i[0]] = x_interp.size
                
                timearray[i[0],0,pop] = auxil2time-auxil1time
                timearray[i[0],1,pop] = auxil3time-auxil2time
                timearray[i[0],2,pop] = auxil4time-auxil3time
                
                timearray_average[i[0],0]  += timearray[i[0],0,pop]
                timearray_average[i[0],1]  += timearray[i[0],1,pop]
                timearray_average[i[0],2]  += timearray[i[0],2,pop]
                
                print("... Time Akima Fortran: {0:} s ".format(auxil2time-auxil1time))
                print("... Time Akima   Spline: {0:} s ".format(auxil3time-auxil2time))
                print("... Time Scipy     Spline: {0:} s ".format(auxil4time-auxil3time))
            
            timearray_average[i[0],0]  /= population
            timearray_average[i[0],1]  /= population 
            timearray_average[i[0],2]  /= population 
            
            standard_deviation[i[0],0] = np.std( timearray[i[0],0,:] )
            standard_deviation[i[0],1] = np.std( timearray[i[0],1,:] )
            standard_deviation[i[0],2] = np.std( timearray[i[0],2,:] )
    
        #for i in range(population):
        #    plt.plot(sizepoints,timearray[:,0,i],linewidth=2,color='darkorange',alpha=0.5)
        #    plt.plot(sizepoints,timearray[:,1,i],linewidth=2,color='darkgreen',alpha=0.5)
        #   plt.plot(sizepoints,timearray[:,2,i],linewidth=2,color='#1f77b4',alpha=0.5)

            
        plt.plot(sizepoints,timearray_average[:,0],color='darkorange',label='Time Akima Fortran', zorder=1)
        #plt.errorbar(sizepoints,timearray_average[:,0], yerr=3.*standard_deviation[:,0],color='orange',ecolor = 'orange', elinewidth = 1, capsize=2, zorder=2)
        #plt.scatter(sizepoints,timearray_average[:,0],color='red', s=2, zorder=3)

        plt.plot(sizepoints,timearray_average[:,1],color='darkgreen',label='Time Akima Scipy', zorder=4)
        #plt.errorbar(sizepoints,timearray_average[:,1], yerr=3.*standard_deviation[:,1],color='green',ecolor = 'lightgreen', elinewidth = 1, capsize=2, zorder=5)
        #plt.scatter(sizepoints,timearray_average[:,1],color='green', s=2, zorder=6)
        
        plt.plot(sizepoints,timearray_average[:,2],color='#1f77b4',label='Time Scipy  Spline')
        #plt.errorbar(sizepoints,timearray_average[:,2], yerr=3.*standard_deviation[:,2],color='blue',ecolor = 'blue', elinewidth = 1, capsize=2, zorder=2)
        #plt.scatter(sizepoints,timearray_average[:,2],color='cyan', s=2, zorder=3)

        plt.xlabel('Number of points in array')
        plt.ylabel('Time [s]')        

        plt.xscale("log")
        plt.yscale("log")

        plt.legend() 
        plt.show()

# Python3 interface for calling the Akima functions
def akima_interp( x, xpt, ypt, delta_x=0.1 ):
    """Convenience method for those who don't want derivatives and don't want to evaluate the same spline multiple times"""
    #print("Python interface")
    
    i = np.argsort(xpt)

    xpt = np.array(xpt[i])
    ypt = np.array(ypt[i])
 
    # Take only unique values 
    # In case they repeat verify if they have the same value and give warning
    i_repeat = np.unique(xpt, return_index=True)[1]

    if i_repeat.size != xpt.size:
        #print("... Houston, we have a problem!")
        xpt = xpt[i_repeat]
        ypt = ypt[i_repeat]

    #print("i_repeat: ",i_repeat)

    p0, p1, p2, p3 = interp.akima_setup( xpt, ypt, delta__x=delta_x )
    #print( p0,p1,p2,p3)

    #y = x*0.0
    #return y       
 
    npt = len(xpt)
    zeros = np.zeros( [npt-1,npt] )

    i_repeat = np.unique(x, return_index=True)[1]
    
    #y = x*0.0
    #return y       
    
    # It should work as well, but the size of the matrices increase the memory exponentially
    #yy_value,dy____dx,dy_dxold,dy_dyold = akimainterp(xx_value,xold_vec,p0,p1,p2,p3   ,dp0dxold,dp1dxold,dp2dxold,dp3dxold,dp0dyold,dp1dyold,dp2dyold,dp3dyold,[is_deriv,is_index])
    #y, dydx, dydxpt, dydypt              = interp.akimainterp( x,            xpt,         p0, p1, p2, p3, zeros,        zeros,      zeros,       zeros,        zeros,       zeros,       zeros,       zeros )

    y, IsKeepOn = interp.akimaspline( x, xpt, ypt, delta__x=delta_x )

    return y

def akima_interp_with_derivs( xpt, ypt, x, delta_x=0.1 ):

    a = Akima( xpt, ypt, delta_x )

    return a.interp(x)

def main():
    i_object = PyAkimaSpline
    i_object.TestRoutine( delta_x=0.1 )
    
    #i_object = PyAkimaSpline    
    #i_object.time()
    
    # x = np.arange(0,2.0*np.pi,0.02)
    # y = np.sin(x**2)
    
    # x_new = [0.5,0.6,0.7,0.8,0.9,1.0,1.0,1.1,1.2,1.22,1.25,1.34,2.3,3.22222,4.66,5.2,5.6,5.7,5.8,6.0]
    
    # i_object = PyAkimaSpline( x_new,x,y,tipo=1 )
        
    # # #print( i_object.y_new )
    
    # o_object = PyAkimaSpline( x,y,delta_x=.400 )
    # a = o_object.interp( x_new )
        
    # #i_object.plot()
    
    # plt.plot( x,y )
    # #plt.plot( x_new,i_object.y_new,color='darkorange',linewidth=2 )
    # plt.plot( x_new,a[0],color='darkgreen',linewidth=2 )
    
if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Revised interface on Sat Jan 30 12:07:21 2021

@author: Jean Gomes

RESUME : FluxConSpec subroutine is written in fortran 2003+ standard. Flux -conserving 
         interpolation script.

Version: v01 beta

PYTHON : Python compatibility using f2py revised. Better usage  with numpy.  

Written: Jean Michel Gomes © Copyright
"""

# Import libraries
# Be careful with importation of pylab - It needs to be first otherwise problems with other features
from pylab import *
import matplotlib.pyplot as plt
import matplotlib as mpl
import matplotlib.pylab as pl

import numpy as np
import random
import time

from PyFluxconserving import flib as interp
from PyFluxconserving import califa_cmap_alternative as califa_cmap
from PyFluxconserving import fluxconserve

print("Welcome to PyFluxConSpec")
try:
    from spectres  import spectral_resampling as sr
    from spectres import spectral_resampling_numba as srn
except:
    print("Third-party packages")
    print("spectres is not installed ==> pip install spectres")
    print("However, not necessary for the runs")
    
import scipy.interpolate as interpolate
from scipy.interpolate import CubicSpline
from scipy.interpolate import BSpline

class PyFluxConSpec(object):
    '''Created on Last version on Wed Sep 23 14:33:51 2020

    @author: Jean Gomes Copyright (c)

    *** Number : 002
    1) FluxConSpec
    2) author_FluxConSpec
    
######################################################################
SUBROUTINE FluxConSpec( Orlambda,Orfluxes,Nrlambda,O_lambda,O_fluxes,                               &
                        N_lambda,per_bins,slow_int,IsKeepOn,fill_val,                                                                 &
                        verbosity )
fluxconspec in  python

RESUME : Rebinning of input vector on a new grid and returns the
                  output array. This routine uses the derivative of a          
                  cumulative function to guarantee the conservation of        
                  density flux while rebinning the spectrum.                   
                                                                           
     OBS.  :  Central wavelengths must be specified to compute the         
                  wavelengths at the edges of the pixels. This routine         
                  assumes a monotonically increasing function for 'X',         
                  that can be unequally spaced.                                
                                                                           
                  The Cumulative function has several ways to be               
                  interpolated:                                                
                                                                           
                  Interpolation Schemes:                                       
                  00) slow_int -> AkimaSpline
                  01) slow_int -> Interpolado
                  02) slow_int -> LINdexerpol
                  03) slow_int -> LINinterpol
                  04) slow_int -> SPLINE1DArr
                  05) slow_int -> SPLINE3DArr
                  06) slow_int -> SPLINECubic

    
Input                    arguments = 7 
Output                 arguments = 2
Optional              arguments = 2
Total number of arguments = 11
   
  INPUT  : 01) Orlambda  -> New 'X' vector                      
                 02)  Nrlambda  -> Number of elements in new 'X' vector       
                 03) O_lambda   -> Old 'X' vector (Abscissa)                    
                 04) O_fluxes      -> Old 'Y' vector (Ordinate)
                 05) N_lambda   -> Number of elements in old 'X' vector         
                 06) per_bins     -> [0: Not conserve flux , 1: Conserve flux  ]  
                 07) slow_int      -> [0 -- 7]                                     
                 08) fill_val         -> Value to fill values outside boundary             
                 09) verbosity   -> Optional variable to print & check           
                                                                           
OUTPUT: 01) Orfluxes -> New 'Y' vector                               
                 02) IsKeepOn -> Flag, if == 0 then there's a problem         
                                                                           
PYTHON: Python compatibility using f2py revised. Better usage        
                  with numpy.                                                  
                                                                           
EXTRA ROUTINES : Those mentioned in slow_int interpolation scheme.    
                                                                           
       LOG : Modified to take into account Y negative values                  
                  and minor corrections in the recovered Y values                  
                  Two different corrections: One related to eps values in the      
                  cumulative function and another one related to the cumulative    
                  being set erroneously to zero.                                   
                                                                           
Checked: Sat Dec  8 12:03:44 WET  2012                                
                 Fri Dec 28 15:34:27 WET  2012                                
                 Wed Apr 29 18:13:43 WEST 2015                                
                 Thu Oct 20 21:55:02 WEST 2016
######################################################################             

######################################################################
SUBROUTINE author_FluxConSpec( a )
author_fluxconspec in python
######################################################################             

######################################################################             
                 HOW TO USE
                 
                 i_object = FluxConSpec
                 i_object.FluxConSpec(x_new,x,y,[per_bins],[slow_int],[fill_val],[verbosity])
                 
                 where
                 
                 x_new is the new array of points (abscissa) to be flux-conserved
                 x,y are the original arrays (abscissa and ordinate)
                 
                 [per_bins]  => optional - [0: Not conserve flux , 1: Conserve flux  ]
                 [slow_int]   => optional - (Type of interpolation - see above)
                 [fill_val]      => optional - Value to fill values outside boundary
                 [verbosity] => verbosity mode [0: No, 1:Yes] 
                 
                 Test ROUTINE
                 
                 i_object = FluxConSpec
                 i_object.TestRoutine()
                 
                 It will produce a plot with the different options for interpolation - Flux-conserving
                 
                 time
                 
                 i_object = FluxConSpec
                 i_object.time()
                 
                 It will test the time it takes to interpolate as a function of elements in the array
######################################################################
'''
    
    def __init__(self,x_new,x,y,per_bins=1,slow_int=0,fill_val=0.0,verbosity=0):
        self.x = x
        self.y = y
        self.x_new = x_new
        
        self.slow_int = slow_int
        self.fill_val = fill_val
        
        self.y_new,self.IsKeepOn =  interp.fluxconspec(self.x_new,self.x,self.y,per_bins,slow_int,fill_val,verbosity)
        
        #print(self.y_new)
        return
    
    def plot( self ):
        string_f = "slow_int = {0:}".format(self.slow_int)
        
        plt.plot(self.x,self.y,label='Original Values')
        plt.plot(self.x_new,self.y_new,label='Interpolated Values - ' + string_f)
        
        plt.xlabel('x')
        plt.ylabel('y')
        
        plt.legend()
        plt.title('Original versus New Interpolated Values')
        plt.show()
    
    def TestRoutine( z=5.52, fill_val = 1.256 ):
        print("... Create a sine wave function")
        x1 = np.arange(-0.5,np.pi/2.0,0.23)
        x2 = np.arange(np.pi/2.0,np.pi,0.010)
        x3 = np.arange(np.pi,2.0*np.pi+0.8+0.2,0.080)

        np.random.seed(100)
        x1 = -0.5  + (0.23 + 0.5) *  np.random.rand(10)
        x1 = np.sort(x1)

        #print("x1",x1)
        x = np.concatenate([x1,x2,x3])
        x = x / (1. + z)
        #x = np.concatenate([x1,x2])
        y = np.where( (x>=0.0) & (x <= 2.0*np.pi), np.sin(x)**4 + 2., 1.0)
        
        #print(x,y)
        #print()
        #ind = np.arange(0,np.size(x),1)
        #random.shuffle(ind)
        
        #x = x[ind]
        #y = y[ind]
        
        # ny_value = lininterpol(nx_value,xold_vec,yold_vec,ilastval,[verbosity])
        verbosity = 0
        IsKeepOn = 1
        slow_int = 0
        per_bins = 1
        
        #fill_val = 1.256
        #fill_val = 0.0
        
        #x_new = [x[0]-0.1,x[0],0.5,1.0,2.3,3.22222,4.66,5.2,6.0,x[-1],x[-1]+0.1]
        #y_new = interp.fluxconspec(x_new,x,y,per_bins,slow_int,IsKeepOn,verbosity)
        
        #print("... Interpolation of array: {0:}".format(x_new))
        
        x_new_ = np.arange(6.5,8.0,0.001)
        x_new_1 = np.arange(4.67,5.2,0.0123)

        x_new = [x[0]-0.7,x[0]-0.1,x[0],0.5,1.0,2.3,np.pi,3.22222,4.66,5.2,6.0,6.2]#,6.5]#,7.0,7.2,7.5,8.0] #,x[-1]+0.5,x[-1]+1.0,x[-1]+2.0]
        x_new = np.concatenate([x_new,x_new_1,x_new_])
        #x_new = np.arange(x[0]-1.0,x[-1]+1.0,0.3)
        x_new = np.array(x_new, dtype=float)
        x_new = np.unique(x_new)
        #x_new = np.arange(x[0]-0.5,x[-1]+0.5,0.5)
        
        begin_time = time.time()
        #print(fill_val)

        y_new_,IsKeepOn = interp.fluxconspec(x_new, x, y, per_bins, slow_int, fill_val=fill_val, verbosity=verbosity)

        try:
            auxil1time = time.time()
            #print(x.size,x)
            model_resampled = sr.spectres(x_new, x, y, fill=fill_val)
            #model_resampled = y_new
            auxil2time = time.time()
        except:
            print("spectres is not installed - Comparison not made")
            
        print("... Time fluxconspec: {}s and spectral_resampling: {}s - Ratio fluxconspec/sample_resampling {}".format(auxil1time-begin_time,auxil2time-auxil1time,(auxil1time-begin_time)/(auxil2time-auxil1time)))
        print()
        
        t_new = np.linspace(x_new[0],x_new[-1], 31)
        h_new = np.interp(t_new,x_new,y_new_)
        
        int0 = np.trapz(y_new_,x=x_new)
        int1 = np.trapz(h_new,x=t_new,dx=(t_new[1]-t_new[0]))
        int2 = np.trapz(model_resampled,x=x_new)
        int3 = np.trapz(y,x=x)
 
        print("... Total areas in the resampling: Flux Conserving {} | Linear Interpolation {} | Spectral Resampling {} | Original Data {}".format(int0,int1,int2,int3))
        print("")
        
        # Types of interpolation used in the cumulative curve
        #interpolation_names = [']
                               
        interpolation_names = [ 'Akima Interpolation       ', 
                                'Linear    Interpolation    ', 
                                'Linear  Table Indexing    ', 
                                'Simple Interpolation      ', 
                                'One Dimensional Spline ', 
                                'SPLINE 3D Array             ', ]
        N_slows = len(interpolation_names)
        
        # Compute Areas
        i_to_integrate = np.where( (x_new >= x[0]) & (x_new <= x[-1]) )
        
        i_to_integrate_original = np.where( (x >= x_new[0]) & (x <= x_new[-1]) )
        A_original = np.trapz(y[i_to_integrate_original],x=x[i_to_integrate_original])
        
        #for i in enumerate(d):
        # y_to_integrate = d[i[1]]
        # A = np.trapz(y_to_integrate[i_to_integrate],x=x_new[i_to_integrate])
        # print(A,int3,A/int3)

        d = {}
        for i in range(0,N_slows):
            slow_int = i
            #print(i,string_f)
            IsKeepOn = 1
            y_new,IsKeepOn = interp.fluxconspec(x_new,x,y,per_bins,slow_int,fill_val=fill_val,verbosity=verbosity)

            if IsKeepOn == 0:
                print("Deu Merda {}".format(slow_int))

            #if i == 3:
            #    print(x_new)
            #    print(y_new)

            y_to_integrate = y_new
            A = np.trapz(y_to_integrate[i_to_integrate],x=x_new[i_to_integrate])
            #print(A,int3,A/int3)
            
            string_ = "{0:} | A =".format(interpolation_names[i])
            #print(len(string_))
            
            string_f = "y_new_{0:} = {1:} {2:<12.6f}".format(i,string_,A)
            d[string_f] = y_new,A

        # Compute Area from simple python script
        model_resampled_fluxconserve = fluxconserve.fluxconserve( x, y, x_new, kind='linear', fill_val=fill_val )
        A = np.trapz(model_resampled_fluxconserve[i_to_integrate],x=x_new[i_to_integrate])
        string_f = "fluxconserve.py in  Python script      | A = {0:<12.6f}".format(A)
        d[string_f] = model_resampled_fluxconserve,A
        
        # Compute Area from Spectral Resampling
        A = np.trapz(model_resampled[i_to_integrate],x=x_new[i_to_integrate])
        #print(A,int3,A/int3)
        string_f = "Spectral Resampling Python - Adam | A = {0:<12.6f}".format(A)
        d[string_f] = model_resampled,A

        #print(d)
        #print("... Interpolation of array:")
        #print(x_new)
        print()
        #print(d)
        
        Figure = plt.figure( figsize=(12,10),dpi=120,facecolor='w',edgecolor='w' )
        plt.subplots_adjust(bottom=.02, left=.06, right=.95, top=.98, wspace=0.0, hspace=0.0) 
        
        ax = plt.subplot(111)

        # Top plot ###########################################################
        ax1_top = subplot2grid( (20,20), (0,0), colspan=20, rowspan=20 )                                                 # Sets the position and size of the panel for Plot #01
        ax1_top.axis('on')
        ax1_top.axes.get_xaxis().set_visible(True)
        
        xmin  = -1.8
        xmax = +10.7
        
        ylimmin = -0.2
        ylimmax = +5.95
        ax1_top.set_xlim( xmin,xmax )
        ax1_top.set_ylim( ylimmin,ylimmax )

        # Original values
        ax1_top.plot(x,y,color='darkorange',linewidth=3,label='Original A = {}'.format(A_original),zorder=1)
        ax1_top.scatter(x,y,color='green',s=100.0,label='Original',zorder=2)
        
        # Interpolated function Fortran - f2py
        #ax1_top.plot(x_interp, y_interp, color='blue',label='Spline 3D Interpolation',linewidth=10,alpha=0.3)

        cmap =  califa_cmap.get_califa_velocity_cmap()
        norm = mpl.colors.Normalize(vmin=0, vmax=len(d)-1)
        colors = cmap(norm(np.arange(0, len(d)+1, 1)))
        for i in enumerate(d):
            #print(i[1])
            
            if i[0] < len(d)-1:
                ax1_top.plot(x_new,d[i[1]][0]+0.3*(i[0]+2),color=colors[i[0]],linewidth=3,label='{}'.format(i[1]),zorder=i[0]+2)
            else:
                ax1_top.plot(x_new,d[i[1]][0]+0.3,color='purple',linewidth=3,linestyle='--',label='{}'.format(i[1]),zorder=i[0]+2)
                ax1_top.scatter(x_new,d[i[1]][0]+0.3,color='purple',s=100.0,label='{}'.format(i[1]),zorder=i[0]+3)

        ax1_top.fill_between(x_new, y_new_, 0.0, facecolor='orange', alpha=0.6, edgecolor='white', hatch='/',zorder=0)

        # Interpolated function Python3 interface
        #ax1_top.plot(t_new, h_new, color='green',linestyle='--',linewidth=4,label='Numpy Python3 Interface')

        # Interpolated function using scipy
        #ax1_top.plot(x_interp, y_interp_scipy,color='black',linewidth=1.2,label='Akima Scipy')

        # Spectral Resampling
        #plt.plot(x_new, model_resampled, color='purple',linewidth=3,linestyle='--',label='Spectral Resampling Python - Adam')

        # Plot points corresponding to the x values
        #ax1_top.scatter(x,y*0,color='cyan',s=100.0)
        #ax1_top.scatter(x_new,y_new*0,color='magenta',s=30.0)

        for i in enumerate(x):
             x2 = [x[i[0]],x[i[0]]]
             y2 = [0,y[i[0]]]
             ax1_top.plot( x2,y2, marker = 'o',color='green' )

        for i in enumerate(x_new):
            x1 = [x_new[i[0]],x_new[i[0]]]
            y1 = [0,y_new_[i[0]]]
            
            ax1_top.plot( x1,y1, marker = 'o',color='black',alpha=0.3 )

        #get handles and labels
        handles, labels = plt.gca().get_legend_handles_labels()

        #print(handles)
        #print(labels)

        #specify order of items in legend
        order1 = np.array( [0,1], dtype=int )
        order2 = np.arange(2,N_slows+5,1)
        order = np.concatenate( [order1,order2], dtype=int )
        
        #add legend to plot
        ax1_top.legend( [handles[idx] for idx in order],[labels[idx] for idx in order],title = "Flux Conserving Interpolations",fancybox=True, shadow=True,loc='upper right' ).set_zorder(1200)

        #ax1_top.legend()
        #ax1_top.set_xlabel("Radians", fontsize=18)
        #ax1_top.set_ylabel("f(x) = sin(x)", fontsize=18)
        
        plt.xlabel("Radians", fontsize=18)
        plt.ylabel("f(x) = sin(x)", fontsize=18)
        # Top plot ###########################################################
        
        # #
        # For some reason if not using bbox_inches='tight' it cuts off labels
        plt.savefig('fluxconserving.png', bbox_inches="tight")
        plt.show()
        
        return
    
    def time():
        print("... Create a sine wave function")
        print("... x and y values for x with equal step size")
     
        x = np.arange(0,4.3*np.pi,0.1)
        y = np.sin(x)
     
        print("... step x: {0:}".format(x[1]-x[0]))

        # No verbosity 
        verbosity = 0
     
        delta_x = 0.1

        #i_ = 25

        #sizepoints = np.zeros(i_)
        #timearray  = np.zeros([i_,3])

        np.random.seed(100)
        
        #x_interp_ = np.random.normal(0.0,1.0,2**i_)
        #x_interp_ = np.unique(x_interp_)
        
        val1 = np.arange(2,10,1)
        val2 = np.arange(10,100,2)
        val3 = np.arange(100,1000,10)
        val4 = np.arange(1000,10000,100)
        val5 = np.arange(10000,100000,1000)
        val6 = np.arange(100000,1000000,10000)
        val7 = np.arange(1000000,10000000,1000000)

        val_array = np.concatenate([val1,val2,val3,val4,val5])#,val6,val7])
        val_array = np.unique(val_array)
        val_array = np.sort(val_array)
    
        x_interp_ = 0. + 2.3*np.pi * (  np.random.normal(0.0,1.0,val_array[-1])  )
        #x_interp_ = np.arange(0,2.3*np.pi,0.001)
        x_interp_ = np.unique(x_interp_)
        x_interp_ = np.sort(x_interp_)
       
        per_bins = 1
        fill_val = -2.3
       
        population = 30
        
        i_ = val_array.size
        sizepoints = np.zeros(i_)
                
        interpolation_names = [ 'Akima Interpolation       ', 
                                'Linear    Interpolation    ', 
                                'Linear  Table Indexing    ', 
                                'Simple Interpolation      ', 
                                'One Dimensional Spline ', 
                                'SPLINE 3D Array             ',
                                'Akima Spline interpolation',
                                'Cubic Spline using scipy',
                                'Spectres',
                                'Spectres Numba',
                                'Numpy Interpolation',]
        
        N_slows = len(interpolation_names)
        
        colors          = pl.cm.jet(np.linspace(0,1,N_slows))
        
        timearray  = np.zeros([i_,N_slows,population])
        timearray_average  = np.zeros([i_,N_slows])
        standard_deviation = np.zeros([i_,N_slows]) 
        
        for i in enumerate(val_array):
            value = val_array[i[0]]
            x_interp = x_interp_[:value]
            print(x_interp[:value].size)
            sizepoints[i[0]] = x_interp.size         
              
            for j in range(0,N_slows-4):
                for pop in range(0,population): 
                    slow_int = j
                    auxil1time = time.time()
                    y_interp,IsKeepOn = interp.fluxconspec(x_interp,x,y,per_bins,slow_int,fill_val=fill_val,verbosity=verbosity)
                    auxil2time = time.time()

                    timearray[i[0],j,pop] = auxil2time-auxil1time
                    timearray_average[i[0],j]  += timearray[i[0],j,pop]
                    #print("{}".format(auxil2time-auxil1time))
            
                timearray_average[i[0],j]  /= population
                standard_deviation[i[0],j] = np.std( timearray[i[0],j,:] )
                
            for pop in range(0,population): 
                x_interp = x_interp_[:value]

                # Calling Akima Spline interpolation
                auxil1time = time.time()
                #y_interp_Akima,IsKeepOn = interp1.akimaspline( x_interp, x, y, delta__x=delta_x, verbosity=verbosity )
                scipy_akima_object = interpolate.Akima1DInterpolator( x,y )
                y_interp_scipy = scipy_akima_object( x_interp )
                auxil2time = time.time()

                # Cubic Spline using scipy
                auxil3time = time.time()
                cs          = CubicSpline( x,y )
                y_interp_scipy = cs(x_interp)
                auxil4time = time.time()

                try:
                    # Spectres
                    auxil5time = time.time()
                    model_resampled = sr.spectres(x_interp, x, y, fill=fill_val)
                    auxil6time = time.time()
                
                    # Spectres Numba
                    auxil7time = time.time()
                    model_resampled = srn.spectres_numba(x_interp, x, y, fill=fill_val)
                    auxil8time = time.time()
                except:
                    print("spectres is not installed - Comparison not made")
                    
                # Numpy Interpolation
                auxil9time = time.time()
                numpy_interp = np.interp(x_interp, x, y)
                auxil10time = time.time()
                
                j = N_slows-5
                timearray[i[0],j,pop] = auxil2time-auxil1time
                timearray_average[i[0],j]  += timearray[i[0],j,pop]
                
                j = N_slows-4
                timearray[i[0],j,pop] = auxil4time-auxil3time
                timearray_average[i[0],j]  += timearray[i[0],j,pop]

                j = N_slows-3
                timearray[i[0],j,pop] = auxil6time-auxil5time
                timearray_average[i[0],j]  += timearray[i[0],j,pop]

                j = N_slows-2
                timearray[i[0],j,pop] = auxil8time-auxil7time
                timearray_average[i[0],j]  += timearray[i[0],j,pop]
                
                j = N_slows-1
                timearray[i[0],j,pop] = auxil10time-auxil9time
                timearray_average[i[0],j]  += timearray[i[0],j,pop]

            timearray_average[i[0], N_slows-5]  /= population
            timearray_average[i[0], N_slows-4]  /= population
            timearray_average[i[0], N_slows-3]  /= population
            timearray_average[i[0], N_slows-2]  /= population
            timearray_average[i[0], N_slows-1]  /= population
            
            standard_deviation[i[0], N_slows-5] = np.std( timearray[i[0], N_slows-5,:] )
            standard_deviation[i[0], N_slows-4] = np.std( timearray[i[0], N_slows-4,:] )
            standard_deviation[i[0], N_slows-3] = np.std( timearray[i[0], N_slows-3,:] )
            standard_deviation[i[0], N_slows-2] = np.std( timearray[i[0], N_slows-2,:] )
            standard_deviation[i[0], N_slows-1] = np.std( timearray[i[0], N_slows-1,:] )

            
        for j in range(0,N_slows):
            #if j==N_slows-1:
            #    plt.plot(sizepoints[:],timearray_average[:,j],label='Time : {0:} {1:}'.format(j,interpolation_names[j]), zorder=1,color='black')
            #else:
            plt.plot(sizepoints[:],timearray_average[:,j],label='Time : {0:} {1:}'.format(j,interpolation_names[j]), zorder=1,color=colors[j])

                
        plt.xlabel('Number of points in array')
        plt.ylabel('Time [s]')        

        plt.xscale("log")
        plt.yscale("log")

        plt.legend( fontsize=7 ) 
        plt.show() 
        
        return 

def main():
    i_object = PyFluxConSpec
    
    # TestRoutine
    #i_object.TestRoutine( z=0.27 )
    
    # time
    i_object.time()
    
    #x = np.arange(0,2.0*np.pi,0.2)
    #y = np.sin(x**2)
    
    #x_new = [0.5,1.0,2.3,3.22222,4.66,5.2,6.0]
    
    #i_object = PyFluxConSpec(x_new,x,y,slow_int=4)
    #i_object.plot()
    #print( i_object.y_new )

    # plt.plot(x,y,color='darkorange')
    # plt.plot(x_new,i_object.y_new)

if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Feb  1 18:52:15 2021

@author: jean
"""

import numpy as np
import scipy.interpolate as sinterp

from PyFluxconserving.flib import lininterpol as interp1
from PyFluxconserving.flib import spline1darr as interp2
from PyFluxconserving.flib import interpolado as interp3
from PyFluxconserving.flib import lindexerpol as interp4

import PyFluxconserving.flib as interp1

#import LINinterpol as interp1
#import SPLINE1DArr as interp2
#import Interpolado as interp3
#import LINdexerpol as interp4

#import interp            as interp5

import matplotlib.pyplot as plt

import time

class Linear__int(object):
    
    def __init__(self,x_new,x,y,e=1.0e-8,verbosity=0):
        self.x = x
        self.y = y
        self.x_new = x_new
        
        self.y_new = interp2.spline1darr(self.x_new,self.x,self.y,e,verbosity)
        
        #print(self.y_new)
    
    def TestRoutine():
        print("... Cerate a sine wave")
        
        x1 = np.arange(0,np.pi,0.2)
        x2 = np.arange(np.pi,2.0*np.pi,0.33)
        #x = np.arange(0,2.0*np.pi,0.2)
        x = np.concatenate([x1,x2])
        y = np.sin(x)

        # ny_value = lininterpol(nx_value,xold_vec,yold_vec,ilastval,[verbosity])
        ilastval = -999
        verbosity = 0
        e         = 1.0e-12
        checking  = 0

        x_new = [x[0]-0.1,x[0],0.5,1.0,2.3,3.22222,4.66,5.2,6.0,x[-1],x[-1]+0.1]

        y_new_interp1, ilatval = interp1.lininterpol(x_new, x, y, ilastval, verbosity)
        y_new_interp2          = interp2.spline1darr(x_new, x, y,        e, verbosity)
        y_new_interp3          = interp3.interpolado(x_new, x, y          , verbosity)
        y_new_interp4          = interp4.lindexerpol(x_new, x, y          , verbosity)
        #y_new_interp5          = interp5.interp_1lin(x_new, x, y, checking, verbosity)
        finterp = sinterp.interp1d(x, y, kind="slinear", fill_value="extrapolate")
        y_new_scipy = finterp(x_new)

        print("... Interpolation of array: {0:}".format(x_new))
        print("... Interpolation LINinterpol: {}".format(np.max(np.abs(y_new_scipy - y_new_interp1))))
        print("... Interpolation SPLINE1DArr: {}".format(np.max(np.abs(y_new_scipy - y_new_interp2))))
        print("... Interpolation Interpolado: {}".format(np.max(np.abs(y_new_scipy - y_new_interp3))))
        print("... Interpolation LINdexerpol: {}".format(np.max(np.abs(y_new_scipy - y_new_interp4))))
        #print("... Interpolation interp_1lin: {}".format(np.max(np.abs(y_new_scipy - y_new_interp5))))

        x_new = [x[0]-3.167,x[0]-2.5,x[0]-0.5,x[0]-0.1,x[0],0.5,1.0,2.3,3.22222,4.66,5.2,6.0,x[-1],x[-1]+0.1,x[-1]+0.5,x[-1]+1.75]

        checking = 0
        ilastval = -999
        y_new_interp1, ilastval = interp1.lininterpol(x_new, x, y, ilastval, verbosity)
        y_new_interp2           = interp2.spline1darr(x_new, x, y,        e, verbosity)
        y_new_interp3           = interp3.interpolado(x_new, x, y          , verbosity)
        y_new_interp4           = interp4.lindexerpol(x_new, x, y          , verbosity)
        #y_new_interp5           = interp5.interp_1lin(x_new, x, y, checking, verbosity)

        # Test with numpy
        #y_new_numpy = np.interp(x_new,x,y,period=2*np.pi)
        finterp = sinterp.interp1d(x,y,kind="slinear",fill_value="extrapolate")
        y_new_scipy = finterp(x_new)

        print("... Interpolation of array: {0:}".format(x_new))
        
        print("... Interpolation LINinterpol: {}".format(np.max(np.abs(y_new_scipy-y_new_interp1))))
        print("... Interpolation SPLINE1DArr: {}".format(np.max(np.abs(y_new_scipy-y_new_interp2))))
        print("... Interpolation Interpolado: {}".format(np.max(np.abs(y_new_scipy-y_new_interp3))))
        print("... Interpolation LINdexerpol: {}".format(np.max(np.abs(y_new_scipy-y_new_interp4))))
        #print("... Interpolation interp_1lin: {}".format(np.max(np.abs(y_new_scipy-y_new_interp5))))

        plt.plot(x,y,linewidth=10)

        plt.plot(x_new, y_new_interp1,color='magenta',linewidth=6)
        plt.plot(x_new, y_new_interp2,color='orange' ,linewidth=9,alpha=0.7)
        plt.plot(x_new, y_new_interp3,color='green'  ,linewidth=5,alpha=0.5)
        plt.plot(x_new, y_new_interp4,color='blue'  ,linewidth=2,alpha=0.5)
        #plt.plot(x_new, y_new_interp5, color='violet', linewidth=2, alpha=0.5)

        #plt.plot(x_new,y_new_numpy,'cyan')
        plt.plot(x_new, y_new_scipy, 'cyan',alpha=0.2)

        plt.scatter(x_new,y_new_interp1,c='red',s=200,alpha=1.0)

        plt.xlabel("radians")
        plt.ylabel("sin(x)")

        plt.show()

    def TestCPU( Nruns=10 ):
        x1 = np.arange(0, np.pi, 0.2)
        x2 = np.arange(np.pi, 2.0 * np.pi, 0.33)
        # x = np.arange(0,2.0*np.pi,0.2)
        x = np.concatenate([x1, x2])
        y = np.sin(x)

        delta = 0.000001
        x_new = np.arange(-1.0,x2[-1]+1.0,delta)

        verbosity = 0
        e = 1.0e-8

        t = np.zeros([7], dtype='float')

        i = np.random.randint(0,x_new.size,)


        start_lininterpol = time.time()
        for i in range(Nruns):
            ilastval = -999
            y_new_interp1, ilastval, IsKeepOn = interp1.lininterpol(x_new, x, y, ilastval, verbosity)
        end_lininterpol = time.time()
        print("... Time LINinterpol: {:12.5}s".format(end_lininterpol-start_lininterpol))
        t[0] = end_lininterpol-start_lininterpol

        start_spline1darr = time.time()
        for i in range(Nruns):
            y_new_interp2, IsKeepOn           = interp2.spline1darr(x_new, x, y, e, verbosity)
        end_spline1darr = time.time()
        print("... Time SPLINE1DArr: {:12.5}s".format(end_spline1darr-start_spline1darr))
        t[1] = end_spline1darr-start_spline1darr

        start_interpolado = time.time()
        for i in range(Nruns):
            y_new_interp3, IsKeepOn           = interp3.interpolado(x_new, x, y, verbosity)
        end_interpolado = time.time()
        print("... Time Interpolado: {:12.5}s".format(end_interpolado - start_interpolado))
        t[2] = end_interpolado - start_interpolado

        start_lindexerpol = time.time()
        for i in range(Nruns):
            y_new_interp4, IsKeepOn           = interp4.lindexerpol(x_new, x, y, verbosity)
        end_lindexerpol = time.time()
        print("... Time LINdexerpol: {:12.5}s".format(end_lindexerpol - start_lindexerpol))
        t[3] = end_lindexerpol - start_lindexerpol

        start_interp_1lin = time.time()
        checking = 0
        #for i in range(Nruns):
        #    y_new_interp4 = interp5.interp_1lin(x_new, x, y, checking, verbosity)
        end_interp_1lin = time.time()
        print("... Time interp_1lin: {:12.5}s".format(end_interp_1lin - start_interp_1lin))
        t[4] = end_interp_1lin - start_interp_1lin

        start_scipy = time.time()
        for i in range(Nruns):
            finterp = sinterp.interp1d(x, y, kind="slinear", fill_value="extrapolate")
            y_new_scipy = finterp(x_new)
        end_scipy = time.time()
        print("... Time scipyinterp: {:12.5}s".format(end_scipy - start_scipy))
        t[5] = end_scipy - start_scipy

        start_numpy = time.time()
        for i in range(Nruns):
            y_new_numpy = np.interp(x_new,x,y)
        end_numpy = time.time()
        print("... Time numpyinterp: {:12.5}s".format(end_numpy - start_numpy))
        t[6] = end_numpy - start_numpy

        print(t[1:]/t[0])

        print(x.size,y.size)

        plt.plot(x, y, linewidth=15)

        plt.plot(x_new, y_new_interp1, color='magenta', linewidth=6)
        plt.plot(x_new, y_new_interp2, color='orange', linewidth=9, alpha=0.7)
        plt.plot(x_new, y_new_interp3, color='green', linewidth=5, alpha=0.5)
        plt.plot(x_new, y_new_interp4, color='blue', linewidth=2, alpha=0.5)
        #plt.plot(x_new, y_new_interp4, color='violet', linewidth=2, alpha=0.5)

        # plt.plot(x_new,y_new_numpy,'cyan')
        #plt.plot(x_new, y_new_scipy, 'cyan', alpha=0.8)

        #plt.scatter(x_new, y_new_interp1, c='red', s=200, alpha=1.0)

        plt.xlabel("radians")
        plt.ylabel("sin(x)")

        plt.show()


def main():
    i_object = Linear__int
    #i_object.TestRoutine()
    
    i_object.TestCPU( Nruns=500 )

if __name__ == "__main__":
    main()



from setuptools import setup, find_packages
from setuptools.command.build_ext import build_ext
import subprocess
import sys
import os
import glob
import shutil

# --- version handling from version.txt ----------------
def read_version():
    here = os.path.abspath(os.path.dirname(__file__))
    version_file = os.path.join(here, "version.txt")
    with open(version_file, "r", encoding="utf-8") as f:
        return f.read().strip()
# --- version handling from version.txt ----------------

PACKAGE_NAME = "PyFluxconserving"
MODULE_NAME = "flib"

class F2PyBuild(build_ext):
    def run(self):
        # Diretório do pacote Python
        pkg_dir = os.path.join(os.getcwd(), PACKAGE_NAME)
        os.makedirs(pkg_dir, exist_ok=True)
        init_file = os.path.join(pkg_dir, "__init__.py")
        if not os.path.exists(init_file):
            open(init_file, "a").close()  # cria __init__.py vazio

        # Lista de fontes Fortran
        sources = [
            'src/fortran/DataTypes.f90',
            'src/fortran/AkimaSpline.f90',
            'src/fortran/Interpolado.f90',
            'src/fortran/LINdexerpol.f90',
            'src/fortran/LINinterpol.f90',
            'src/fortran/SPLINE1DArr.f90',
            'src/fortran/SPLINE3DFor.f90',
            'src/fortran/FluxConSpec.f90',
        ]

        # --- Gerar .pyf interface ---
        cmd = [sys.executable, "-m", "numpy.f2py", "-m", MODULE_NAME, "-h",
               os.path.join(pkg_dir, f"{MODULE_NAME}.pyf")] + sources
        subprocess.check_call(cmd)

        # --- Compilar Fortran para .so dentro do pacote ---
        cmd = [sys.executable, "-m", "numpy.f2py", "-c",
               os.path.join(pkg_dir, f"{MODULE_NAME}.pyf")] + sources
        subprocess.check_call(cmd)

        super().run()

        # --- Mover o .so para dentro do pacote, se não estiver ---
        for so_file in glob.glob(f"{MODULE_NAME}*.so"):
            shutil.move(so_file, os.path.join(pkg_dir, so_file))


        # --- Copiar arquivos Python para dentro do pacote ---
        py_src_dir = os.path.join(os.getcwd(), "src", "python")
        if os.path.isdir(py_src_dir):
            for py_file in glob.glob(os.path.join(py_src_dir, "*.py")):
                shutil.copy(py_file, pkg_dir)

# Setup principal
setup(
    name=PACKAGE_NAME,
    version=read_version(),
    packages=[PACKAGE_NAME],
    #packages=find_packages(),  # busca pacotes em src/python
    #package_dir={"": "src/python"},              # root do pacote é src/python
    cmdclass={"build_ext": F2PyBuild},
    #install_requires=["numpy", "scipy", "matplotlib"],  # dependências
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Revised interface on Sat Jan 30 12:07:21 2021

@author: Jean Gomes

RESUME : Spline3DFor subroutine is written in Fortran 2003+ standard.

Version: v01 beta

PYTHON : Python compatibility using f2py revised. Better usage  with numpy.  

Written: Jean Michel Gomes © Copyright ® 
 
Date: Sun Oct 15 10:00:52 WEST 2006
"""

# Import libraries
# Be careful with importation of pylab - It needs to be first otherwise problems with other features
from pylab import *
import matplotlib.pyplot as plt

import numpy as np

from PyFluxconserving import flib as interp

import scipy.interpolate as interpolate
from scipy.interpolate import CubicSpline
from scipy.interpolate import BSpline

class SPLINE3DClass(object):
    '''Created on 
Last version on Wed Sep 23 14:33:51 2020

@author: Jean Gomes Copyright (c)

*** Number : 005 Fortran Routines
1) SPLINE3DCoe
2) I_SPLINE3D
3) SPLINE3DFor
4) SPLINE3DArr
5) author_SPLINE3DFor

######################################################################
SUBROUTINE SPLINE3DCoe( x,y,b,c,d,n,IsKeepOn,verbosity )
spline3dcoe in python

RESUME : Interpolation with cubic spline function. Evaluate the 
                  coefficients b(i), c(i), and d(i), i=1,2,...,n for cubic spline 
                  interpolation. In mathematics, a spline is a special function   
                  defined piecewise by polynomials. In interpolating problems,    
                  spline interpolation is often preferred to polynomial           
                  interpolation because it yields similar results, even when using
                  low degree polynomials, while avoiding Runge's phenomenon for   
                  higher degrees.                                                 
                                                                
                  s(x) = y(i) + b(i)*(x-x(i))**1 + c(i)*(x-x(i))**2  + d(i)*(x-x(i))**3                         
                                                                
                  For x(i) <= x <= x(i+1), strictly valid otherwise extrapolate.                                           
                                                                
Input                    arguments = 3                                   
Output                 arguments = 4                                   
Optional              arguments = 1                                   
Total number of arguments = 8                                   
 
   INPUT  : 01) x                -> Old x vector (abcissas)               
                  02) y                 -> Old y vector (ordenadas)              
                  03) n                 -> # of elements in vector x and y       
                  04) verbosity   -> Print & Check screen                  
                  
OUTPUT : 01) b                 -> Spline coefficient order 1                    
                  02)  c                 -> Spline coefficient order 2                    
                  03)  d                -> Spline coefficient order 3                    
                  04)  IsKeepOn -> Flag, if == 0 then there's a problem
                                                                
PYTHON : Python compatibility using f2py revised. Better usage  
                  with numpy.                                            
                                                                
References :                                                    
1) Forsythe, G.E. (1977) Computer Methods For Mathematical      
Computations. Ed. Prentice-Hall, Inc.                           
                                                                
2) Bartels et al. (1998)                                        
                                                                
Written: Jean Michel Gomes © Copyright ®                        
Checked: Tue May  1 16:09:13 WEST 2012                          
                  Fri Dec 28 14:55:10 WET  2012                          
######################################################################

######################################################################      
REAL (KIND=RP) FUNCTION I_SPLINE3D( u,x,y,b,c,d,n )
i_spline3d in python

RESUME : This function evaluates the cubic spline interpolation       
                  at point u, i.e.                                             
                                                                           
                  I_SPLINE3D = y(i) + b(i)*(u-x(i))**1 + c(i)*(u-x(i))**2      
                                       + d(i)*(u-x(i))**3                                
                                                                           
                  Where x(i) <= u <= x(i+1)                                    
                                                                           
                 The coefficients b(i), c(i), and d(i), i=1,2,...,n for       
                 cubic spline interpolation is computed in the spline         
                 subroutine.                                                  

Input                   arguments = 7                                         
Output                arguments = 1                                         
Optional              arguments = 0                                         
Total number of arguments = 8                                         

   INPUT  : 01) u          -> New x value (abcissa)                      
                   02) x          -> Old x vector (abcissas)                    
                   03) y          -> Old y vector (ordenadas)                   
                   04) b          -> Spline coefficient order 1                 
                   05) c          -> Spline coefficient order 2                 
                   06) d          -> Spline coefficient order 3                 
                   07) n          -> # of elements in vector x and y            
                                                                           
OUTPUT : 01) I_SPLINE3D -> Interpolated value at u                    
                                                                           
PYTHON : Python compatibility using f2py revised. Better usage        
                  with numpy.                                                  
                                                                           
Written: Jean Michel Gomes                                            
Checked: Tue May  1 16:09:13 WEST 2012                                
                  Fri Dec 28 14:55:10 WET  2012                                
######################################################################      

######################################################################      
REAL (KIND=RP) FUNCTION SPLINE3DFor( u,x,y,n,IsKeepOn,verbosity )
spline3Dfor in python

RESUME :  This function evaluates the cubic spline interpolation       
                    at u (real value), i.e.                                      
                                                                           
                   SPLINE3DFor = y(i) + b(i)*(u-x(i))**1 + c(i)*(u-x(i))**2
                                           + d(i)*(u-x(i))**3

                   Where stricly valid from x(i) <= u <= x(i+1)                 
                                                                           
                   The coefficients b(i), c(i), and d(i), i=1,2,...,n for       
                   cubic spline interpolation is computed in the spline         
                   subroutine.                                                  
                                                                                                                                                    
Input                   arguments = 4                                         
Output                arguments = 2                                         
Optional              arguments = 1                                         
Total number of arguments = 7                                         
                                                                           
   INPUT  : 01) u           -> New x value (abcissa)                     
                  02) x           -> Old x vector (abcissas)                   
                  03) y           -> Old y vector (ordenadas)                  
                  04) n           -> # of elements in vector x and y           
                  05) verbosity   -> Print & Check screen                      
                                                                           
OUTPUT : 01) SPLINE3DFor -> Interpolated value of y at point u
                  02) IsKeepOn    -> Flag, if == 0 then there's a problem      
                                                                           
PYTHON : Python compatibility using f2py revised. Better usage        
                  with numpy.                                                  
                                                                           
Written: Jean Michel Gomes                                            
Checked: Tue May  1 16:09:13 WEST 2012                                
                  Fri Dec 28 14:55:10 WET  2012                                
######################################################################      

######################################################################
SUBROUTINE SPLINE3DArr( u,o,m,x,y,n,IsKeepOn,verbosity )
spline3darr in python

RESUME : This function evaluates the cubic spline interpolation       
                  for an array u (j going from 0 to m), i.e.                   
                                                                          
                  SPLINE3DArr = y(i) + b(i)*(u(j)-x(i))**1                     
                                                    + c(i)*(u(j)-x(i))**2                     
                                                    + d(i)*(u(j)-x(i))**3                     
                                                                           
                  Where stricly valid from x(i) <= u(j) <= x(i+1)              
                                                                           
                  The coefficients b(i), c(i), and d(i), i=1,2,...,n for       
                  cubic spline interpolation is computed in the spline         
                  subroutine.                                                  
                                                                         
Input                   arguments = 5                                         
Output                arguments = 2                                         
Optional              arguments = 1                                         
Total number of arguments = 8                                         
         
   INPUT  : 01) u                -> New x value (abcissa)                     
                  02) m               -> # of elements in vector u and o           
                  03) x                 -> Old x vector (abcissas)                   
                  04) y                 -> Old y vector (ordenadas)                  
                  05) n                 -> # of elements in vector x and y           
                  06) verbosity   -> Print & Check screen                      
                                                                           
OUTPUT : 01) o                 -> Interpolated array of ys at points u      
                  02) IsKeepOn  -> Flag, if == 0 then there's a problem      
                                                                           
PYTHON : Python compatibility using f2py revised. Better usage        
                  with numpy.                                                  
                                                                                       
Written: Jean Michel Gomes                                            
Checked: Tue May  1 16:09:13 WEST 2012                                
                  Fri Dec 28 14:55:10 WET  2012                                
######################################################################

######################################################################      
SUBROUTINE author_SPLINE3DFor( a )
author_spline3dfor in python
######################################################################      

######################################################################             
                 HOW TO USE
                 
                 i_object = SPLINE3DClass
                 i_object.SPLINE3DClass(x_new,x,y,[verbosity])
                 
                 where
                 
                 x_new is the new array of points (abscissa) to be flux-conserved
                 x,y are the original arrays (abscissa and ordinate)
                 
                 [verbosity] => verbosity mode [0: No, 1:Yes] 
                 
                 Test ROUTINE
                 
                 i_object = SPLINE3DClass
                 i_object.TestRoutine()
                 
                 It will produce a plot with the sine interpolation
                 
                 time
                 
                 i_object = SPLINE3DClass
                 i_object.time()
                 
                 It will test the time it takes to interpolate as a function of elements in the array
######################################################################
'''
    def __init__(self,x_new,x,y,e=1.0e-8,verbosity=0):
        self.x = x
        self.y = y
        self.x_new = x_new
        
        self.y_new,self.IsKeepOn = interp.spline3darr(self.x_new,self.x,self.y,verbosity)
        
        #print(self.y_new)
    
    def TestRoutine():
        print("... Create a sine wave function")
        print("... x and y values for x with equal step size")
        
        x = np.arange(0,2.3*np.pi,1.0)
        y = np.sin(x)
        
        #print(x.size)

        print("... step x: {0:}".format(x[1]-x[0]))

        # No verbosity 
        verbosity = 0
        
        print("... x and y values for x with unequal step size")
        
        #x_new = [x[0]-0.1,x[0],0.5,1.0,2.3,3.22222,4.66,5.2,6.0,x[-1],x[-1]+0.1]
        x_new = [x[0]-0.2,x[0]-0.1,x[0],0.5,1.0,1.5,2.3,3.22222,4.66,5.2,6.0,x[-1],x[-1]+0.1,x[-1]+0.5]
        x_new = np.array( x_new, dtype=float )
        x_new = np.unique(x_new)
        y_new = np.sin(x_new)
        
        dxnew = x_new[1:] - x_new[:-1]
        
        print("... step x_new: {0:}".format(dxnew))
                
        #x_new = [x[0]-0.5,x[0]-0.1,x[0],0.5,1.0,1.5,2.3,3.22222,4.66,5.2,6.0,x[-1],x[-1]+0.1,x[-1]+0.5]
        
        x_interp          = np.arange(x[0]-1.89,x_new[-1]+1.5,0.01)
        #x_interp = np.array( [-0.7,-0.7,-0.7,-0.7,-0.7099999,0.2,0.3,0.3,0.9,1.2,1.2,1.5,2.3,2.3,3.4,3.5], dtype=float )
        x_new_interp = x_interp
    
        print("... Interpolation of array: {0:}".format([x_interp[0],x_interp[-1]]))

        # Calling directly the f2py subroutine
        b,c,d,IsKeepOn = interp.spline3dcoe(x,y)
        #print(b,c,d)
        b_new,c_new,d_new,IsKeepOn = interp.spline3dcoe(x_new,y_new)
        #print(b_new,c_new,d_new)
        
        y_interp,IsKeepOn = interp.spline3darr(x_interp,x,y,verbosity=verbosity)
        y_new_interp,IsKeepOn = interp.spline3darr(x_new_interp,x_new,y_new,verbosity=verbosity)
        
        #y_interp = interp.spline3darr(x_interp,x,y)
        #y_new_interp = interp.spline3darr(x_new_interp,x_new,y_new)

        # Calling Akima Spline interpolation
        delta_x = 0.1
        y_interp_Akima,IsKeepOn = interp.akimaspline( x_interp, x, y, delta__x=delta_x, verbosity=verbosity )
        y_new_interp_Akima,IsKeepOn = interp.akimaspline( x_new_interp, x_new, y_new, delta__x=delta_x, verbosity=verbosity )

        # Cubic Spline using scipy
        cs          = CubicSpline( x,y,bc_type='natural' )
        #cs          = CubicSpline( x,y, bc_type='not-a-knot' )
        #cs          = CubicSpline( x,y,bc_type='clamped' )
        #cs          = CubicSpline( x,y,bc_type='periodic' )
        #spl        = interpolate.splrep(x,y)
        #print(spl[0])
        #print(spl[1])
        #print(spl[2])
        
        k            =  3
        t, c, k         = interpolate.splrep(x, y, s=0, k=k)
        
        # x_new must be striclty increasing
        x_new_ = np.unique(x_new)
        y_new_ = np.sin(x_new_)
        cs_new = CubicSpline( x_new_,y_new_ )
        spl_new = BSpline( x,y,k)
        #print('y_new_interp:', y_new_interp)

        Figure = plt.figure( figsize=(12,10),dpi=120,facecolor='w',edgecolor='w' )
        plt.subplots_adjust(bottom=.02, left=.06, right=.95, top=.98, wspace=0.0, hspace=0.0) 
        
        ax = plt.subplot(111)

        # Top plot ###########################################################
        ax1_top = subplot2grid( (20,20), (0,0), colspan=20, rowspan=10 )                                                 # Sets the position and size of the panel for Plot #01
        ax1_top.axis('on')
        ax1_top.axes.get_xaxis().set_visible(False)

        ylimmin = -3.9
        ylimmax =+4.9 # max( y_interp ) * 1.3
        #ax1_top.set_xlim( xmin,xmax )
        ax1_top.set_ylim( ylimmin,ylimmax )

        # Original values
        ax1_top.plot(x,y,color='darkorange',linewidth=10,label='Original')
        
        # Interpolated function Fortran - f2py
        ax1_top.plot(x_interp, y_interp, color='blue',label='Spline 3D Interpolation',linewidth=10,alpha=0.3)
        ax1_top.plot(x_interp, y_interp_Akima, color='magenta',label='Akima Spline Interpolation',linestyle='--',linewidth=4,alpha=1.)
        ax1_top.scatter(x_interp, y_interp*0.)

        # Interpolated function using scipy
        ax1_top.plot(x_interp, cs(x_interp), color='black',linestyle='-',linewidth=1.2,label='Scipy Cubic Spline')
        #spline = BSpline(t, c, k, extrapolate=True )
        #ax1_top.plot(x_interp, spline(x_interp), color='black',linestyle='-',linewidth=1.2,label='Scipy Cubic Spline')

        ax1_top.legend()
        ax1_top.set_xlabel("Radians")
        ax1_top.set_ylabel("f(x) = sin(x)")
        # Top plot ###########################################################

        # Bottom plot ########################################################
        ax2_top = subplot2grid( (20,20), (10,0), colspan=20, rowspan=9 )                                                # Sets the position and size of the panel for Plot #02
        ax2_top.axis('on')
        
        # Original Different Point Values
        ax2_top.plot(x_new,y_new,color='darkorange',label='Original Diff Points',linewidth=10)
        
        # Interpolated function Fortran - f2py
        ax2_top.plot(x_new_interp, y_new_interp, color='blue',label='Spline 3D Interpolation',linewidth=10,alpha=0.3)
        ax2_top.plot(x_new_interp, y_new_interp_Akima, color='magenta',label='Akima Spline Interpolation',linestyle='--',linewidth=4,alpha=1.)
        ax2_top.scatter(x_new_interp, y_new_interp*0.)

        # # Interpolated function Python3 interface
        # ax2_top.plot(x_new_interp, y_new_interp_Akima, color='magenta',linestyle='--',linewidth=4,label='Akima Python3 Interface')

        # Interpolated function using scipy
        ax2_top.plot(x_new_interp, cs_new(x_new_interp), color='black',linestyle='-',linewidth=1.2,label='Scipy Cubic Spline')
        #ax2_top.plot(x_new_interp, spl_new(x_new_interp), color='black',linestyle='-',linewidth=1.2,label='Scipy Cubic Spline')

        # ax2_top.legend()
        # ax2_top.set_xlabel("Radians")
        # ax2_top.set_ylabel("f(x) = sin(x)")
        
        plt.show()
        # Bottom plot ########################################################

    def time():
        print("... Create a sine wave function")
        print("... x and y values for x with equal step size")
     
        x = np.arange(0,2.3*np.pi,0.05)
        y = np.sin(x)
     
        print("... step x: {0:}".format(x[1]-x[0]))

        # No verbosity 
        verbosity = 0
     
        delta_x = 0.1

        #i_ = 25

        #sizepoints = np.zeros(i_)
        #timearray  = np.zeros([i_,3])

        np.random.seed(10)
        
        #x_interp_ = np.random.normal(0.0,1.0,2**i_)
        #x_interp_ = np.unique(x_interp_)
        
        val1 = np.arange(1,10,1)
        val2 = np.arange(10,100,2)
        val3 = np.arange(100,1000,10)
        val4 = np.arange(1000,10000,100)
        val5 = np.arange(10000,100000,1000)
        val6 = np.arange(100000,1000000,10000)
        val7 = np.arange(1000000,10000000,1000000)

        val_array = np.concatenate([val1,val2,val3,val4,val5,val6,val7])
        val_array = np.unique(val_array)
        
        #print(val_array)
        #return
    
        x_interp_ = 0. + 2.3*np.pi * (  np.random.normal(0.0,1.0,val_array[-1])  )
        x_interp_ = np.unique(x_interp_)
       
        i_ = val_array.size
        sizepoints = np.zeros(i_)
       
        population = 10
        timearray  = np.zeros([i_,3,population])
        timearray_average  = np.zeros([i_,3])
        standard_deviation = np.zeros([i_,3])
       
        value = 1
        for i in enumerate(val_array):
           
            for pop in range(0,population):
                value = val_array[i[0]]
                x_interp = x_interp_[0:value]
                
                print("... iteration: {}".format(i[0]))
                print("... Interpolation of array: {0:} {1:} with size == {2:}".format(x_interp[0],x_interp[-1],x_interp.size))
                
                # Calling directly the f2py subroutine
                auxil1time = time.time()
                y_interp,IsKeepOn = interp.spline3darr(x_interp,x,y,verbosity=verbosity)
                #y_interp, IsKeepOn = interp.akimaspline( x_interp, x, y, delta__x=delta_x, verbosity=verbosity )
               
                # Calling Akima Spline interpolation
                auxil2time = time.time()
                #y_interp_Akima,IsKeepOn = interp.akimaspline( x_interp, x, y, delta__x=delta_x, verbosity=verbosity )
                scipy_akima_object = interpolate.Akima1DInterpolator( x,y )
                y_interp_scipy = scipy_akima_object( x_interp )
                
                # Cubic Spline using scipy
                auxil3time = time.time()
                cs          = CubicSpline( x,y )
                y_interp_scipy = cs(x_interp)
                
                auxil4time = time.time()
                
                sizepoints[i[0]] = x_interp.size
                
                timearray[i[0],0,pop] = auxil2time-auxil1time
                timearray[i[0],1,pop] = auxil3time-auxil2time
                timearray[i[0],2,pop] = auxil4time-auxil3time
                
                timearray_average[i[0],0]  += timearray[i[0],0,pop]
                timearray_average[i[0],1]  += timearray[i[0],1,pop]
                timearray_average[i[0],2]  += timearray[i[0],2,pop]
                
                print("... Time    Spline3DFor: {0:} s ".format(auxil2time-auxil1time))
                print("... Time Akima   Spline: {0:} s ".format(auxil3time-auxil2time))
                print("... Time Scipy     Spline: {0:} s ".format(auxil4time-auxil3time))
                
            timearray_average[i[0],0]  /= population
            timearray_average[i[0],1]  /= population 
            timearray_average[i[0],2]  /= population 
                
            standard_deviation[i[0],0] = np.std( timearray[i[0],0,:] )
            standard_deviation[i[0],1] = np.std( timearray[i[0],1,:] )
            standard_deviation[i[0],2] = np.std( timearray[i[0],2,:] )
                
            #for i in range(population):
            #    plt.plot(sizepoints,timearray[:,0,i],linewidth=2,color='darkorange',alpha=0.5)
            #    lt.plot(sizepoints,timearray[:,1,i],linewidth=2,color='darkgreen',alpha=0.5)
            #    plt.plot(sizepoints,timearray[:,2,i],linewidth=2,color='#1f77b4',alpha=0.5)

           
        plt.plot(sizepoints,timearray_average[:,0],color='darkorange',label='Time Akima Fortran', zorder=1)
        #plt.errorbar(sizepoints,timearray_average[:,0], yerr=3.*standard_deviation[:,0],color='orange',ecolor = 'orange', elinewidth = 1, capsize=2, zorder=2)
        #plt.scatter(sizepoints,timearray_average[:,0],color='red', s=2, zorder=3)

        plt.plot(sizepoints,timearray_average[:,1],color='darkgreen',label='Time Akima Scipy', zorder=4)
        #plt.errorbar(sizepoints,timearray_average[:,1], yerr=3.*standard_deviation[:,1],color='green',ecolor = 'lightgreen', elinewidth = 1, capsize=2, zorder=5)
        #plt.scatter(sizepoints,timearray_average[:,1],color='green', s=2, zorder=6)
       
        plt.plot(sizepoints,timearray_average[:,2],color='#1f77b4',label='Time Scipy  Spline')
        #plt.errorbar(sizepoints,timearray_average[:,2], yerr=3.*standard_deviation[:,2],color='blue',ecolor = 'blue', elinewidth = 1, capsize=2, zorder=2)
        #plt.scatter(sizepoints,timearray_average[:,2],color='cyan', s=2, zorder=3)

        plt.xlabel('Number of points in array')
        plt.ylabel('Time [s]')        

        plt.xscale("log")
        plt.yscale("log")

        plt.legend() 
        plt.show()

def main():
    i_object = SPLINE3DClass
    
    i_object.TestRoutine()
    
    #i_object.time()
    
    #x = np.arange(0,2.0*np.pi,0.2)
    #y = np.sin(x**2)
    
    #x_new = [0.5,1.0,2.3,3.22222,4.66,5.2,6.0]
    
    #i_object = SPLINE3DFor( x_new,x,y )
    #print( i_object.y_new )

if __name__ == "__main__":
    main()
